"use strict";
// deno-lint-ignore-file no-explicit-any
Object.defineProperty(exports, "__esModule", { value: true });
exports.evaluate = exports.shift = exports.reset = void 0;
function* reset(block) {
    return yield { type: "reset", block };
}
exports.reset = reset;
function* shift(block) {
    return yield { type: "shift", block };
}
exports.shift = shift;
function evaluate(block, getNext = $next()) {
    let stack = [block()];
    let value;
    for (let current = stack.pop(); current; current = stack.pop()) {
        let prog = current[Symbol.iterator]();
        try {
            let next = getNext(prog);
            getNext = (iter) => iter.next(value);
            if (next.done) {
                value = next.value;
            }
            else {
                let cont = ({ [Symbol.iterator]: () => prog });
                let control = next.value;
                if (control.type === "reset") {
                    stack.push(cont);
                    stack.push(control.block());
                }
                else {
                    let resolve = oneshot((value) => evaluate(() => cont, $next(value)));
                    let reject = oneshot((error) => evaluate(() => cont, $throw(error)));
                    stack.push(control.block(resolve, reject));
                }
            }
        }
        catch (error) {
            if (!stack.length) {
                throw error;
            }
            else {
                getNext = $throw(error);
            }
        }
    }
    return value;
}
exports.evaluate = evaluate;
function oneshot(fn) {
    let continued = false;
    let result;
    return ((value) => {
        if (!continued) {
            continued = true;
            return result = fn(value);
        }
        else {
            return result;
        }
    });
}
const $next = (value) => (i) => i.next(value);
const $throw = (error) => (i) => {
    if (i.throw) {
        return i.throw(error);
    }
    else {
        throw error;
    }
};
//# sourceMappingURL=mod.js.map