"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createChannel = void 0;
const instructions_js_1 = require("./instructions.js");
function createChannel() {
    let subscribers = new Set();
    let output = {
        [Symbol.iterator]: () => (0, instructions_js_1.resource)(function* (provide) {
            let subscriber = {
                notify() { },
            };
            let subscription = {
                [Symbol.iterator]: () => (0, instructions_js_1.action)(function* (resolve) {
                    subscriber.notify = resolve;
                    yield* (0, instructions_js_1.suspend)();
                })[Symbol.iterator](),
            };
            subscribers.add(subscriber);
            try {
                yield* provide(subscription);
            }
            finally {
                subscribers.delete(subscriber);
            }
        })[Symbol.iterator](),
    };
    let send = (item) => {
        return (0, instructions_js_1.action)(function* (resolve, reject) {
            yield* [{
                    type: "suspend",
                    then() {
                        let result = { type: "resolved", value: void 0 };
                        for (let subscriber of subscribers) {
                            try {
                                subscriber.notify(item);
                            }
                            catch (error) {
                                result = { type: "rejected", error };
                            }
                        }
                        if (result.type === "rejected") {
                            reject(result.error);
                        }
                        else {
                            resolve(result.value);
                        }
                    },
                }];
        });
    };
    let input = {
        send: (value) => send({ done: false, value }),
        close: (value) => send({ done: true, value }),
    };
    return { input, output };
}
exports.createChannel = createChannel;
//# sourceMappingURL=channel.js.map