"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createFuture = void 0;
const instructions_js_1 = require("./instructions.js");
const lazy_js_1 = require("./lazy.js");
function createFuture() {
    let result = null;
    let watchers = new Set();
    let promise = (0, lazy_js_1.lazy)(() => new Promise((resolve, reject) => {
        watchers.add({ resolve, reject });
        notify();
    }));
    function notify() {
        if (result) {
            while (watchers.size > 0) {
                for (let watcher of watchers) {
                    watchers.delete(watcher);
                    if (result.type === "resolved") {
                        watcher.resolve(result.value);
                    }
                    else {
                        watcher.reject(result.error);
                    }
                }
            }
        }
    }
    let settle = {
        resolve(value) {
            result = { type: "resolved", value };
            settle.resolve = settle.reject = () => { };
            notify();
        },
        reject(error) {
            result = { type: "rejected", error };
            settle.resolve = settle.reject = () => { };
            notify();
        },
    };
    let future = {
        [Symbol.toStringTag]: "Future",
        *[Symbol.iterator]() {
            if (result) {
                if (result.type === "resolved") {
                    return result.value;
                }
                else {
                    throw result.error;
                }
            }
            else {
                return yield* (0, instructions_js_1.action)(function* (resolve, reject) {
                    let watcher = { resolve, reject };
                    watchers.add(watcher);
                    try {
                        yield* (0, instructions_js_1.suspend)();
                    }
                    finally {
                        watchers.delete(watcher);
                    }
                });
            }
        },
        then: (...args) => promise().then(...args),
        catch: (...args) => promise().catch(...args),
        finally: (...args) => promise().finally(...args),
    };
    return {
        future,
        resolve: (value) => settle.resolve(value),
        reject: (error) => settle.reject(error),
    };
}
exports.createFuture = createFuture;
//# sourceMappingURL=future.js.map