"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createContext = void 0;
function createContext(name, defaultValue) {
    return {
        [Symbol.toStringTag]: `Context(${name})`,
        *[Symbol.iterator]() {
            let frame = yield { type: "getframe" };
            let context = frame.context[name];
            if (!context) {
                if (typeof defaultValue !== "undefined") {
                    return defaultValue;
                }
                else {
                    throw new MissingContextError(`${context} '${name}' is not available`);
                }
            }
            return context;
        },
    };
}
exports.createContext = createContext;
class MissingContextError extends Error {
    constructor() {
        super(...arguments);
        Object.defineProperty(this, "name", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: "MissingContextError"
        });
    }
}
//# sourceMappingURL=context.js.map