import type { PSMap, PSModule, PSValue } from "./types.js";
import type { Task } from "./deps.js";
import { createYSEnv, global, parse } from "./evaluate.js";
import { concat, createPSMap } from "./psmap.js";
import { load } from "./load.js";
import { run } from "./deps.js";

export interface PlatformScript {
  eval(value: PSValue, bindings?: PSMap): Task<PSValue>;
  load(url: string | URL, base?: string): Task<PSModule>;
  parse(source: string, filename?: string): PSValue;
}

export function createPlatformScript(extensions?: PSMap): PlatformScript {
  let ext = extensions ?? createPSMap();
  let env = createYSEnv(concat(global, ext));

  return {
    eval(value, bindings) {
      return run(() => env.eval(value, bindings));
    },
    load(location, base) {
      return run(() => load({ location, base, env }));
    },
    parse(source, filename) {
      return parse(source, filename);
    },
  };
}
